# -*- coding: utf-8 -*-

from Handlers import BaseHandler
import syslog
#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
class Filter(BaseHandler.BaseHandler):
	#=======================================================
	# 
	#=======================================================
	def __init__(self):
		self.name = 'Filter'
		self.interfaces = ['eth0','eth1','br0','ppp0']
		self.zones = ['external','trusted','internal']
		self.configurable_zones = ['external','internal']
	#=======================================================
	# 
	#=======================================================
	def Get(self, request):

		status = 200
		zone_info = self.zoneInfo()
		zone_info['services'] = self.getService()
		return zone_info, status

	#=======================================================
	# 
	#=======================================================
	def Put(self, request):

		status = 200
		args = request.json
		
		zone_info = self.zoneInfo()
		self.setInterface(args['interfaces'],zone_info['interfaces'])

		zone = args['filters']['zone']
		filter = {zone:{}}
		filter[zone] = args['filters']
		self.setFilter(filter,zone_info['filters'])

		self.setRules(args['rules'])
		return status

	#=======================================================
	# 
	#=======================================================
	def zoneInfo(self):

		zone_info = {'interfaces':{}, 'filters':{}, 'rules':{}, 'zones': self.zones}
		for zone in self.configurable_zones:
			zone_info['filters'][zone] = {'services':[], 'masquerade':'no', 'port':[]}
			zone_info['rules'][zone] = []
		
		result = self.firewallCmd("list-all-zones")
		if result:
			lines = result.split('\n')
			zone = False
			for line in lines:
				if not line.startswith('\t') and  not line.startswith(' '):
					zone = line.split(' ')[0]
					if zone in self.configurable_zones:
						rules_flg = False
						configurable_flg = True
					elif zone in self.zones:
						configurable_flg = False
					else:
						zone = False
				elif zone:
					line = line.strip(' ')
					if line.startswith('interfaces:'):
						interfaces = line.split(':')[1].split(' ')
						for interface in interfaces:
							if interface in self.interfaces:
								zone_info['interfaces'][interface] = zone

					if configurable_flg:
						if line.startswith('services:'):
							services = line.split(':')[1].split(' ')
							for service in services:
								if service:
									zone_info['filters'][zone]['services'].append(service)

						elif line.startswith('ports:'):
							ports = line.split(':')[1].split(' ')
							for port in ports:
								if port:
									zone_info['filters'][zone]['port'].append(port)

						elif line.startswith('masquerade:'):
							masquerade = line.split(':')[1].split(' ')
							zone_info['filters'][zone]['masquerade'] = masquerade[1]

						elif line.startswith('rich rules:'):
							rules_flg = True

						elif rules_flg:
							rule_dict = {'sourceAddr':'', 'destAddr':'', 'port':'', 'protocol':'', 'target':'' }
							if not 'forward' in line:
								rule = line.strip('\t').split(' ')
								if len(rule) > 1:
									src_flg = False
									dst_flg = False
									for param in rule:

										if param.startswith('source'):
											src_flg = True
										elif param.startswith('destination'):
											dst_flg = True
										elif param.startswith('address='):
											if src_flg:
												rule_dict['sourceAddr'] = param.strip('address=').strip('"')
												src_flg = False
											elif dst_flg:
												rule_dict['destAddr'] = param.strip('address=').strip('"')
												dst_flg = False
										elif param.startswith('port='):
											rule_dict['port'] = param.strip('port=').strip('"')
										elif param.startswith('protocol='):
											rule_dict['protocol'] = param.strip('protocol=').strip('"')
										elif param == 'accept' or param == 'reject' or param == 'drop':
											rule_dict['target'] = param
									zone_info['rules'][zone].append(rule_dict)
		print (zone_info)
		return zone_info

	#=======================================================
	# 
	#=======================================================
	def getService(self):

		result = self.firewallCmd("get-services")
		return result.strip('\n').split(' ')

	#=======================================================
	# 
	#=======================================================
	def setInterface(self, args, interfaces):

		for interface in args:
			#zone = self.firewallCmd("get-zone-of-interface=%s" % interface).strip('\n')
			#print( '%s = %s ' % (zone, args[interface]))
			if interfaces[interface] != args[interface]:
				self.firewallCmd("change-interface=%s --zone=%s" % (interface, args[interface]), True)

	#=======================================================
	# 
	#=======================================================
	def setFilter(self, args, filters):

		#print (args)
		for zone in args:
			for key in args[zone]:
				if key == 'services':
					service_list = filters[zone]['services']
					for service in args[zone][key]:
						if service in service_list:
							service_list.remove(service)
						else:
							self.firewallCmd("add-service=%s --zone=%s" % (service, zone), True)
					if service_list:
						for service in service_list:
							self.firewallCmd("remove-service=%s --zone=%s" % (service, zone), True)

				elif key == 'masquerade':
					if args[zone][key] == 'yes' and filters[zone]['masquerade']=='no':
						self.firewallCmd("add-masquerade --zone=%s" % zone, True)
					elif args[zone][key] == 'no' and filters[zone]['masquerade']=='yes':
						self.firewallCmd("remove-masquerade --zone=%s" % zone, True)

	#=======================================================
	# 
	#=======================================================
	def setRules(self, args):

		#print (args)

		for zone in args:
			now_rules = self.firewallCmd("list-rich-rules --zone=%s" % zone).strip('\n').split('\n')
			for rule in args[zone]:
				add_rule = []
				add_rule.append('rule family=\"ipv4\"') 
				if 'sourceAddr' in rule and rule['sourceAddr']:
					add_rule.append('source address=\"%s\"' % rule['sourceAddr'])
				if 'destAddr' in rule and rule['destAddr']:
					add_rule.append('destination address=\"%s\"' % rule['destAddr'])
				if 'port' in rule and rule['port']:
					add_rule.append('port port=\"%s\"' % rule['port'])
				if 'protocol' in rule and rule['protocol']:
					add_rule.append('protocol=\"%s\"' % rule['protocol'])
				if 'target' in rule and rule['target']:
					add_rule.append(rule['target'])

				if len(add_rule) > 2:
					rule_str = ' '.join(add_rule)
					if rule_str in now_rules:
						now_rules.remove(rule_str)
					else:
						self.firewallCmd("add-rich-rule='%s' --zone=%s" % (rule_str, zone), True)
			if now_rules:
				for rule in now_rules:
					if rule and not 'forward' in rule:
						self.firewallCmd("remove-rich-rule='%s' --zone=%s" % (rule, zone), True)

